﻿// Copyright (c) 2019 by E. Lücke

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace G_Api_Wrapper
{
    /// <summary>
    /// This class performs function calls to the g_api.dll file.
    /// The library implements bus independant call to Goepel hardware.
    /// </summary>
    public class GAPI
    {

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // common parameters
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        /// <summary>
        /// Common return code indicating successful execution.
        /// </summary>
        public static UInt32 G_NO_ERROR = 0;

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        [StructLayout(LayoutKind.Explicit)]
        struct G_IpAddress_t
        {
            [FieldOffset(0)]
            public byte B1;

            [FieldOffset(1)]
            public byte B2;

            [FieldOffset(2)]
            public byte B3;

            [FieldOffset(3)]
            public byte B4;

            [FieldOffset(0)]
            public UInt32 All;
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        public enum G_InterfaceType_t : Byte
        {
            UNKNOWN,
            BOOT_LOADER,
            SEQUENCE,
            IO,
            CAN,
            LIN,
            KLINE,
            MOST,
            LVDS,
            FLEXRAY,
            NET2RUN,
            USER_CODE,
            FS,
            RESISTORS,
            ETHERNET,
            COMMON,
            CAN_STM
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        public enum G_HostInterfaceType_t : UInt32
        {
            PCI3051,
            PXI3051,
            PCI3052,
            PXI3052,
            PCI3060,
            PXI3060,
            PCI3072,
            PXI3072,
            PCI3080,
            PXI3080,
            USB_BASICCAN,
            USB3052,
            USB3060,
            USB_BASICLIN,
            USB3072,
            USB_BASICCAR,
            USB3080,
            SMART_CAR_USB,
            ETHERNET,
            USB4120,
            PCI3090,
            PXI3090,
            PXI6153,
            USB6153,
            PXI6173,
            USB6173,
            PXI6191,
            USB6191,
            BASICCAR3085,
            PXI6181,
            USB6181,
            BASIC_CAN_61_PLUS,
            MAGIC_CAR_3,
            USB4009,
            PXI6100,
            PXI3078,
            USB3132,
            USB3116,
            PXI6161,
            USB4115,
            USB6161,
            BASIC_MOST_6161,
            USB4112,
            USB4113,
            BASIC_CAN_6153,
            BASIC_LIN_6173,
            BASIC_CAR_6181,
            BASIC_FLEX_6191,
            PCI6153,
            PCI6173,
            PCI6181,
            PCI6191,
            HUB4X,
            APIX2_ADK__TX,
            APIX2_ADK__RX,
            BASIC_CON_4121,
            PXI4112,
            PXI4113,
            ETH3030,
            ETH3116,
            ETH3118,
            ETH3132,
            ETH3164,
            ETH31128,
            PXI3116,
            PXI3118,
            PXI3132,
            PXI3164,
            PXI31128,
            USB3118,
            USB31128,
            USB3164,
            PXI6141,
            PXI4009,
            GCAR6282,
            ETH4045,
            ETH4009,
            ETH3100,
            GPCIE6222,
            GPCIE6281,
            UNKNOWN
        }



            //**********************************************************************************************************************************************************************
            // USER FUNCTIONS
            //**********************************************************************************************************************************************************************

            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            // G_GetLastErrorDescription
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

            //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
            /// <summary>
            /// G_GetLastErrorDescription
            /// Get error description of last error.
            /// </summary>
            /// <returns>string with error description</returns>
            //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


            [DllImport("g_api.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr G_GetLastErrorDescription();

        public static String Call_G_GetLastErrorDescription()
        {
            IntPtr ptr = G_GetLastErrorDescription();
            return GAPI_Helpers.PtrToStringUtf8(ptr);
        }

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_GetApiVersion
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_GetApiVersion
        /// Returns the current version of the G-API.
        /// </summary>
        /// <param name="major">returns major version</param>
        /// <param name="minor">returns minor version</param>
        /// <param name="revision">returns revision</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__API__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_GetApiVersion(ref UInt32 major, ref UInt32 minor, ref UInt32 revision);

        public static UInt32 Call_G_GetApiVersion(ref UInt32 major, ref UInt32 minor, ref UInt32 revision)
        {
            return G_GetApiVersion(ref major, ref minor, ref revision);
        }

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Interface_GetInterfaceList
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        public enum G_HostCommunicationType_t : Byte
        {
            UNKNOWN,
            PXI,
            PCI,
            USB,
            ETHERNET,
            USB_OVER_ETHERNET,
            SERIAL,
            PCIE
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct G_InterfaceData_t
        {
            public G_InterfaceType_t InterfaceType;
            public Byte ControllerNumber;
            public Byte InterfaceNumber;
            public G_HostCommunicationType_t HostCommunicationType;
            public G_HostInterfaceType_t HostInterfaceType;
            public UInt32 Serial;
            public UInt32 DeviceIndex;
            G_IpAddress_t IpAddress;
            public UInt16 TcpPort;
            public Byte PortNumber;
            public Byte GeographicAddress;
            G_IpAddress_t SubNetMask;
            public UInt32 HostSerial;
            G_IpAddress_t DefaultGateway;
            public UInt32 reserved2;

            public override string ToString()
            {
                String ret;

                ret = String.Format("InterfaceType: {0} \nControllerNumber: {1}\nHostComunicationType: {2}", InterfaceType, ControllerNumber, HostCommunicationType);

                return ret;
            }
        }

        [DllImport("g_api.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Interface_GetInterfaceList(UInt32 cmdFlags, ref UInt32 numberOfInterfaces, ref Byte interfaceData, UInt32 sizeOfInterfaceData);

        public static UInt32 Call_G_Interface_GetInterfaceList(UInt32 cmdFlags, ref UInt32 numberOfInterfaces, ref G_InterfaceData_t[] interfaceData, UInt32 sizeOfInterfaceData)
        {
            Byte[] data = new Byte[3584];

            UInt32 returnValue = G_Interface_GetInterfaceList(cmdFlags, ref numberOfInterfaces, ref data[0], sizeOfInterfaceData);

            // error request
            if (GAPI.G_NO_ERROR == returnValue)
            {
                // no error, compilation from byte array to a structure array
                interfaceData = GAPI_Helpers.ByteArrayToStructureArray<G_InterfaceData_t>(data, numberOfInterfaces, 40);
            }
            else
            {
                // error, so empty array and return
                interfaceData = new G_InterfaceData_t[0];
            }

            // return result from API call as result from call function
            return returnValue;

        }

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Interface_Name_Get
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        [DllImport("g_api.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Interface_Name_Get(ref G_InterfaceData_t interfaceData, ref UInt32 length, ref StringBuilder name);

        public static UInt32 Call_G_Interface_Name_Get(ref G_InterfaceData_t interfaceData, ref UInt32 length, ref StringBuilder name)
        {
            return G_Interface_Name_Get(ref interfaceData, ref length, ref name);
        }

    }
}
