﻿// Copyright (c) 2019 by E. Lücke

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;


namespace G_Api_Wrapper
{
    /// <summary>
    /// This class performs function calls to the g_api_lvds.dll file.
    /// The library provides access to the LVDS hardware functionality.
    /// </summary>
    public class GAPI_LVDS
    {

        //**********************************************************************************************************************************************************************
        // FRAME GENERATOR
        //**********************************************************************************************************************************************************************

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_FrameGenerator_GetFileInfo
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        
        public enum G_Lvds_FrameGenerator_FileType_t : Byte
        {
            UNKNOWN,
            BMP
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct G_Lvds_FrameGenerator_GetFileInfo_Rsp_t
        {
            public G_Lvds_FrameGenerator_FileType_t FileType;
            public Byte reserved;   // must be initialized with 0
            public UInt16 BitDepth; // in bit
            public UInt32 FileSize; // in byte
            public UInt32 Width;    // in px
            public UInt32 Height;   // in px
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_FrameGenerator_GetFileInfo
        /// </summary>
        /// <description>return file information</description>
        /// <param name="portHandle">portHandle</param>
        /// <param name="fileNumber">file number</param>
        /// <param name="rsp">returns file information</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__LVDS__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Lvds_FrameGenerator_GetFileInfo(UInt32 portHandle, Byte fileNumber, ref G_Lvds_FrameGenerator_GetFileInfo_Rsp_t rsp);

        public static UInt32 Call_G_Lvds_FrameGenerator_GetFileInfo(UInt32 portHandle, Byte fileNumber, ref G_Lvds_FrameGenerator_GetFileInfo_Rsp_t rsp)
        {
            return G_Lvds_FrameGenerator_GetFileInfo(portHandle, fileNumber, ref rsp);
        }


        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_FrameGenerator_GetHardwareInfo2
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        public enum G_Lvds_FrameGenerator_Serializer_t : Byte
        {
            UNKNOWN,
            MAX9247,
            MAX9209_MAX9213,
            DS90C241,
            INAP125T24,
            APIX_INAP375T,
            DS90UR905Q,
            DS90UB925Q,
            DS90C385A,
            MAX9275,
            MAX9273,
            MAX9271,
            DS90UB947Q,
            MAX9259,
            DS90UB913,
            RGB888T,
            MAX9295A,
            DS90UB953,
            APIX_INAP375T_MII,
            INAP562T,
            DS90UB971,
            DS90UB981,
            NONE__MIPI_CSI2_TX,
            NONE__MIPI_DSI_TX,
            NONE__OPEN_LDI_TX
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct  G_Lvds_FrameGenerator_GetHardwareInfo2_Rsp_t
        {
            public UInt32 HWInfoFlags;
            public UInt32 PixelClockFrequency;  // in Hz
            public UInt32 H_BlankWidth;         // in clocks
            public UInt32 H_Total;              // in clocks
            public UInt32 H_SyncWidth;          // in clocks
            public UInt32 H_FrontPorch;         // in clocks
            public UInt32 V_BlankWidth;         // in clocks
            public UInt32 V_Total;              // in clocks
            public UInt32 V_SyncWidth;          // in clocks
            public UInt32 V_FrontPorch;         // in clocks
            public UInt16 FpgaVersion;
            public G_Lvds_FrameGenerator_Serializer_t Serializer;
            public Byte reserved;               // must be initialized with 0
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_FrameGenerator_GetHardwareInfo2
        /// </summary>
        /// <description>return hardware information</description>
        /// <param name="portHandle">portHandle</param>
        /// <param name="rsp">returns hardware information</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__LVDS__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Lvds_FrameGenerator_GetHardwareInfo2(UInt32 portHandle, ref G_Lvds_FrameGenerator_GetHardwareInfo2_Rsp_t rsp);

        public static UInt32 Call_G_Lvds_FrameGenerator_GetHardwareInfo2(UInt32 portHandle, ref G_Lvds_FrameGenerator_GetHardwareInfo2_Rsp_t rsp)
        {
            return G_Lvds_FrameGenerator_GetHardwareInfo2(portHandle, ref rsp);
        }


        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_FrameGenerator_GetMemoryInfo
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
                 
        [StructLayout(LayoutKind.Sequential)]
        public struct G_Lvds_FrameGenerator_GetMemoryInfo_Rsp_t
        {
            public UInt32 Flags;
            public UInt32 MemorySize;
            public UInt32 FreeMemory;
            public UInt32 NumberOfFiles;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
            public Byte[] FileNumbers;
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_FrameGenerator_GetMemoryInfo
        /// </summary>
        /// <description>return hardware information</description>
        /// <param name="portHandle">portHandle</param>
        /// <param name="rsp">returns memory information</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__LVDS__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Lvds_FrameGenerator_GetMemoryInfo(UInt32 portHandle, ref G_Lvds_FrameGenerator_GetMemoryInfo_Rsp_t rsp);

        public static UInt32 Call_G_Lvds_FrameGenerator_GetMemoryInfo(UInt32 portHandle, ref G_Lvds_FrameGenerator_GetMemoryInfo_Rsp_t rsp)
        {
            return G_Lvds_FrameGenerator_GetMemoryInfo(portHandle, ref rsp);
        }


        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_FrameGenerator_SetDisplayProperties2
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        
        [StructLayout(LayoutKind.Sequential)]
        public struct G_Lvds_FrameGenerator_SetDisplayProperties2_Cmd_t
        {
            public UInt32 CmdFlags;
            public UInt32 PixelClock;   // dependant on flag "USE_FREQUENCY"
            public UInt32 H_BlankWidth;
            public UInt32 H_Total;
            public UInt32 H_SyncWidth;
            public UInt32 H_FrontPorch;
            public UInt32 V_BlankWidth;
            public UInt32 V_Total;
            public UInt32 V_SyncWidth;
            public UInt32 V_FrontPorch;
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_FrameGenerator_SetDisplayProperties2
        /// </summary>
        /// <description>set display properties</description>
        /// <param name="portHandle">portHandle</param>
        /// <param name="cmd">command parameters</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__LVDS__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Lvds_FrameGenerator_SetDisplayProperties2(UInt32 portHandle, ref G_Lvds_FrameGenerator_SetDisplayProperties2_Cmd_t cmd);

        public static UInt32 Call_G_Lvds_FrameGenerator_SetDisplayProperties2(UInt32 portHandle, ref G_Lvds_FrameGenerator_SetDisplayProperties2_Cmd_t cmd)
        {
            return G_Lvds_FrameGenerator_SetDisplayProperties2(portHandle, ref cmd);
        }

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_Common_Config_Set
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_Common_Config_Set
        /// </summary>
        /// <description>LVDS configuration by configuration file</description>
        /// <param name="portHandle">porthandle</param>
        /// <param name="cmdFlags">command flags</param>
        /// <param name="filename">filename of configuration file</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__COMMON__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Lvds_Common_Config_Set(UInt32 portHandle, UInt32 cmdFlags, StringBuilder filename);

        public static UInt32 Call_G_Lvds_Common_Config_Set(UInt32 portHandle, UInt32 cmdFlags, StringBuilder filename)
        {
            return G_Lvds_Common_Config_Set(portHandle, cmdFlags, filename);
        }

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_I2C_Write_Register_From_File
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        public enum G_Lvds_Common_Data_DataMode_t : Byte
        {
            NONE,
            I2C_MASTER_TO_DESERIALIZER,
            I2C_MASTER_TO_SERIALIZER,
            SPI_MASTER_TO_DESERIALIZER,
            SPI_MASTER_TO_SERIALIZER,
            SPI_PASSTHROUGH_DUAL,
            SPI_RECEIVER_DUAL,
            UART_TO_DESERIALIZER,
            UART_TO_SERIALIZER,
            SPI_TRANSMITTER_DUAL,
            SPI_SLAVE_TO_DESERIALIZER,
            SPI_SLAVE_TO_SERIALIZER,
            I2C_SLAVE_TO_DESERIALIZER,
            I2C_SLAVE_TO_SERIALIZER,
            UNKNOWN,
            INVALID
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_I2C_Write_Register_From_File
        /// </summary>
        /// <description>read given file and write the I2C Register Configuration</description>
        /// <param name="portHandle">porthandle</param>
        /// <param name="dataMode">data mode to set</param>
        /// <param name="path">file to read</param>
        /// <param name="register_written">amount of written registers</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__COMMON__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]

        public static extern UInt32 G_Lvds_I2C_Write_Register_From_File(UInt32 portHandle, G_Lvds_Common_Data_DataMode_t dataMode, StringBuilder path, ref UInt32 register_written);

        public static UInt32 Call_G_Lvds_I2C_Write_Register_From_File(UInt32 portHandle, G_Lvds_Common_Data_DataMode_t dataMode, StringBuilder path, ref UInt32 register_written)
        {
            return G_Lvds_I2C_Write_Register_From_File(portHandle, dataMode, path, ref register_written);
        }

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_FrameGenerator_DisplayBuffer
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        [StructLayout(LayoutKind.Sequential)]
        public struct G_Lvds_FrameGenerator_DisplayBuffer_Cmd_t
        {
            public UInt32 CmdFlags;
            public UInt32 Length;               // data length in bytes
            public UInt16 HorizontalOffset;     // horizontal display offset in pixel
            public UInt16 VerticalOffset;       // vertical display offset in pixel
            public UInt16 HorizontalSize;       // horizontal display size in pixel
            public UInt16 VerticalSize;         // vertical display size in pixel
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_FrameGenerator_DisplayBuffer
        /// </summary>
        /// <description>display buffer data</description>
        /// <param name="portHandle">porthandle</param>
        /// <param name="cmd">command parameters</param>
        /// <param name="data">pointer to data</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__COMMON__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]

        public static extern UInt32 G_Lvds_FrameGenerator_DisplayBuffer(UInt32 portHandle, ref G_Lvds_FrameGenerator_DisplayBuffer_Cmd_t cmd, ref Byte data);

        public static UInt32 Call_G_Lvds_FrameGenerator_DisplayBuffer(UInt32 portHandle, ref G_Lvds_FrameGenerator_DisplayBuffer_Cmd_t cmd, ref Byte data)
        {
            return G_Lvds_FrameGenerator_DisplayBuffer(portHandle, ref cmd, ref data);
        }

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_Common_Data_Register_Write
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        [StructLayout(LayoutKind.Sequential)]
        public struct G_Lvds_Common_Data_Register_Write_Cmd_t
        {
            public UInt32 CmdFlags;
            public UInt16 RegisterAddress;
            public Byte DeviceAddress;
            public Byte RegisterValue;         // disregarded if flag "EXTENDED_DATA" or "REGISTER_BLOCK" is set
            public Byte NumberOfRegisters;       // used if flag "EXTENDED_DATA" or "REGISTER_BLOCK" is set
            public Byte reserved1;
            public Byte reserved2;
            public Byte reserved3;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
            public Byte[] Data;
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_Common_Data_Register_Write
        /// </summary>
        /// <description>write into media interface register</description>
        /// <param name="portHandle">porthandle</param>
        /// <param name="cmd">command parameters</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__COMMON__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]

        public static extern UInt32 G_Lvds_Common_Data_Register_Write(UInt32 portHandle, ref G_Lvds_Common_Data_Register_Write_Cmd_t cmd);

        public static UInt32 Call_G_Lvds_Common_Data_Register_Write(UInt32 portHandle, ref G_Lvds_Common_Data_Register_Write_Cmd_t cmd)
        {
            return G_Lvds_Common_Data_Register_Write(portHandle, ref cmd);
        }

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_Common_Data_I2cTransferEx
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        [StructLayout(LayoutKind.Sequential)]
        public struct G_Lvds_Common_Data_I2cTransferEx_Cmd_t
        {
            public UInt32 CmdFlags;
            public Byte NumberOfTxBytes;
            public Byte NumberOfRxBytes;         
            public Byte SendStartMask;       // not used if flag "NORMAL_READ_WRITE" is set
            public Byte reserved;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
            public Byte[] TxData;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct G_Lvds_Common_Data_I2cTransferEx_Rsp_t
        {
            public UInt32 RspFlags;
            public Byte NumberOfAckBytes;
            public Byte NumberOfRxBytes;
            public Byte reserved1;     
            public Byte reserved2;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
            public Byte[] RxData;
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_Common_Data_I2cTransferEx
        /// </summary>
        /// <description>I2C transfer on the media interface (extended version)</description>
        /// <param name="portHandle">porthandle</param>
        /// <param name="cmd">command parameters</param>
        /// <param name="rsp">returns response parameters</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__COMMON__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]

        public static extern UInt32 G_Lvds_Common_Data_I2cTransferEx(UInt32 portHandle, ref G_Lvds_Common_Data_I2cTransferEx_Cmd_t cmd, ref G_Lvds_Common_Data_I2cTransferEx_Rsp_t rsp);

        public static UInt32 Call_G_Lvds_Common_Data_I2cTransferEx(UInt32 portHandle, ref G_Lvds_Common_Data_I2cTransferEx_Cmd_t cmd, ref G_Lvds_Common_Data_I2cTransferEx_Rsp_t rsp)
        {
            return G_Lvds_Common_Data_I2cTransferEx(portHandle, ref cmd, ref rsp);

        }

    }
}
