﻿// Copyright (c) 2019 by E. Lücke

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using G_Api_Wrapper;

namespace G_Wrapper_Test
{
    class ProgramLVDS
    {
        static void Main(string[] args)
        {
            // Common variables

            String result_str = "";
            UInt32 portNumber = 0;
            StringBuilder interfaceName = new StringBuilder("LVDS1");

            // Struct GenHWInfo
            GAPI_LVDS.G_Lvds_FrameGenerator_GetHardwareInfo2_Rsp_t GenHWInfo = new GAPI_LVDS.G_Lvds_FrameGenerator_GetHardwareInfo2_Rsp_t();


            // get the version of the installed runtime

            UInt32 major = 0;
            UInt32 minor = 0;
            UInt32 revision = 0;

            if (GAPI.G_NO_ERROR != GAPI.Call_G_GetApiVersion(ref major, ref minor, ref revision))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }

            // print the version

            Console.Write("This is GApi version: ");
            Console.Write(major); Console.Write('.');
            Console.Write(minor); Console.Write('.');
            Console.WriteLine(revision);

            // Get Interface List

            UInt32 interfaceflags = 0;
            UInt32 numberOfInterfaces = 128;
            UInt32 sizeOfInterfaceData = 40;
            GAPI.G_InterfaceData_t[] ifData = new GAPI.G_InterfaceData_t[3584];


            if (GAPI.G_NO_ERROR != GAPI.Call_G_Interface_GetInterfaceList(interfaceflags, ref numberOfInterfaces, ref ifData, sizeOfInterfaceData))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }
            else
            {
                Console.WriteLine("Interface List: ");
                Console.WriteLine("Number of Interfaces: " + numberOfInterfaces);
                for (UInt32 i = 0; i < numberOfInterfaces; i++)
                {
                    Console.WriteLine("Interface " + i + ": ");
                    Console.WriteLine(ifData[i]);
                }
            }

            

            // Common_OpenInterface

            if (GAPI.G_NO_ERROR != GAPI_Common.Call_G_Common_OpenInterface(interfaceName, ref portNumber))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }

            Console.WriteLine("PortNumber " + portNumber + " opened");

/*          // Init Interface
            if (GAPI.G_NO_ERROR != GAPI_Common.Call_G_Common_InitInterface(portNumber))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
             }
*/

            // Lvds_FrameGenerator_GetHardwareInfo2

            if (GAPI.G_NO_ERROR == GAPI_LVDS.Call_G_Lvds_FrameGenerator_GetHardwareInfo2(portNumber, ref GenHWInfo))
            {
                Console.WriteLine("Serializer: " + GenHWInfo.Serializer);
                Console.WriteLine("PixelClock: " + GenHWInfo.PixelClockFrequency + "kHz");
            }
            else
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }


/*            // Set Display Properties

            GAPI_LVDS.G_Lvds_FrameGenerator_SetDisplayProperties2_Cmd_t GenSetProp = new GAPI_LVDS.G_Lvds_FrameGenerator_SetDisplayProperties2_Cmd_t();

            GenSetProp.CmdFlags = 0x02;
            GenSetProp.PixelClock = 50000;
            GenSetProp.H_BlankWidth = 206;
            GenSetProp.H_Total = 1056;
            GenSetProp.H_SyncWidth = 32;
            GenSetProp.H_FrontPorch = 18;
            GenSetProp.V_BlankWidth = 32;
            GenSetProp.V_Total = 525;
            GenSetProp.V_SyncWidth = 10;
            GenSetProp.V_FrontPorch = 3;

            if (GAPI.G_NO_ERROR != GAPI_LVDS.Call_G_Lvds_FrameGenerator_SetDisplayProperties2(portNumber, ref GenSetProp))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }
*/

            // Get Memory Info

            GAPI_LVDS.G_Lvds_FrameGenerator_GetMemoryInfo_Rsp_t MemoryInfo = new GAPI_LVDS.G_Lvds_FrameGenerator_GetMemoryInfo_Rsp_t();

            if (GAPI.G_NO_ERROR != GAPI_LVDS.Call_G_Lvds_FrameGenerator_GetMemoryInfo(portNumber, ref MemoryInfo))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }

            Console.WriteLine("Number of Files: " + MemoryInfo.NumberOfFiles);
            for (UInt32 i = 0; i < MemoryInfo.NumberOfFiles; i++)
            {
                
                Console.WriteLine("File Number " + i + ": " + MemoryInfo.FileNumbers[i]);
            }


            // Get File Info

            GAPI_LVDS.G_Lvds_FrameGenerator_GetFileInfo_Rsp_t FileInfo = new GAPI_LVDS.G_Lvds_FrameGenerator_GetFileInfo_Rsp_t();

            Byte fileNumber = 1;

            if (GAPI.G_NO_ERROR != GAPI_LVDS.Call_G_Lvds_FrameGenerator_GetFileInfo(portNumber, fileNumber, ref FileInfo))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }

            Console.WriteLine("File Width: " + FileInfo.Width);
            Console.WriteLine("File Height: " + FileInfo.Height);


            // Last function!
            // Common_CloseInterface

            if (GAPI.G_NO_ERROR != GAPI_Common.Call_G_Common_CloseInterface(portNumber))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }

            Console.WriteLine("PortNumber " + portNumber + " closed");

            Console.WriteLine("Done");
            Console.ReadKey();
        }
    }
}
