﻿// Copyright (c) 2022 by E. Richter
// This example shows the following G-API commands:
// G_GetApiVersion
// G_Common_OpenInterface
// G_Common_InitInterface
// G_Lvds_FrameGenerator_GetHardwareInfo2
// G_Lvds_Common_Config_Set
// G_Lvds_I2C_Write_Register_From_File
// G_Lvds_FrameGenerator_DisplayBuffer
// G_Lvds_Common_Data_Register_Write
// G_Lvds_Common_Data_I2cTransferEx
// G_Common_CloseInterface

// Attention: The Interface name is set to "LVDS9" (line 36)

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using G_Api_Wrapper;

namespace G_Wrapper_Test
{

    class ProgramLVDS_Generate
    {    

        static void Main(string[] args)
        {
            // Common variables

            String result_str = "";
            UInt32 portNumber = 0;
            StringBuilder interfaceName = new StringBuilder("LVDS9");

            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            // get the G-API version of the installed runtime

            UInt32 major = 0;
            UInt32 minor = 0;
            UInt32 revision = 0;

            if (GAPI.G_NO_ERROR != GAPI.Call_G_GetApiVersion(ref major, ref minor, ref revision))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }

            // print the version

            Console.Write("This is GApi version: ");
            Console.Write(major); Console.Write('.');
            Console.Write(minor); Console.Write('.');
            Console.WriteLine(revision);

            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            // Common_OpenInterface

            if (GAPI.G_NO_ERROR != GAPI_Common.Call_G_Common_OpenInterface(interfaceName, ref portNumber))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }

            Console.WriteLine("PortNumber " + portNumber + " opened");

            /*
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            // Init Interface

            if (GAPI.G_NO_ERROR != GAPI_Common.Call_G_Common_InitInterface(portNumber))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
             }
               */         
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            // Lvds_FrameGenerator_GetHardwareInfo2
            /*
            GAPI_LVDS.G_Lvds_FrameGenerator_GetHardwareInfo2_Rsp_t GenHWInfo = new GAPI_LVDS.G_Lvds_FrameGenerator_GetHardwareInfo2_Rsp_t();

            if (GAPI.G_NO_ERROR == GAPI_LVDS.Call_G_Lvds_FrameGenerator_GetHardwareInfo2(portNumber, ref GenHWInfo))
            {
                Console.WriteLine("Serializer: " + GenHWInfo.Serializer);
                Console.WriteLine("PixelClock: " + GenHWInfo.PixelClockFrequency + "kHz");
            }
            else
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }
            */
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            // Lvds_Common_Config_Set

            UInt32 interfaceflags = 0;

            Console.WriteLine("Insert Config path (*.XML):");       // e.g. D:/Temp/Cs/config.xml
            String conffilepath = Console.ReadLine();
            StringBuilder conf_file = new StringBuilder(conffilepath);

            if (GAPI.G_NO_ERROR != GAPI_LVDS.Call_G_Lvds_Common_Config_Set(portNumber, interfaceflags, conf_file))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }
            else
            {
                Console.WriteLine("Config set");
            }
            
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            // Lvds_I2C_Write_Register_From_File

            UInt32 written_register = 0;

            Console.WriteLine("Insert I2C path (*.TXT):");      // e.g. D:/Temp/Cs/I2C_command_sequence.txt
            String i2cfilepath = Console.ReadLine();
            StringBuilder i2c_file = new StringBuilder(i2cfilepath);

            GAPI_LVDS.G_Lvds_Common_Data_DataMode_t mode = new GAPI_LVDS.G_Lvds_Common_Data_DataMode_t();
            //Byte mode = new Byte (2);
            mode = GAPI_LVDS.G_Lvds_Common_Data_DataMode_t.I2C_MASTER_TO_SERIALIZER;

            if (GAPI.G_NO_ERROR != GAPI_LVDS.Call_G_Lvds_I2C_Write_Register_From_File(portNumber, mode, i2c_file, ref written_register))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }
            else
            { 
                Console.WriteLine("I2C commands written");
            }          
            
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            // FrameGenerator Display Buffer

            GAPI_LVDS.G_Lvds_FrameGenerator_DisplayBuffer_Cmd_t DispBuff = new GAPI_LVDS.G_Lvds_FrameGenerator_DisplayBuffer_Cmd_t();

            DispBuff.CmdFlags = 0;
            DispBuff.Length = 7879680; // Heigt x Width x 3
            DispBuff.HorizontalOffset = 0;
            DispBuff.VerticalOffset = 0;
            DispBuff.HorizontalSize = 2432; 
            DispBuff.VerticalSize = 1080;

            Console.WriteLine("Insert image path (*.BMP):");      // e.g. D:/Temp/Cs/colorbar_2432x1080.bmp
            String imagepath = Console.ReadLine();
            System.Drawing.Image image = System.Drawing.Image.FromFile(imagepath);

            byte[] ImageToByteArray(System.Drawing.Image imageIn)
            {
                using (var ms = new System.IO.MemoryStream())
                {
                    imageIn.Save(ms, imageIn.RawFormat);
                    return ms.ToArray();
                }
            }

            byte[] imageBytes = ImageToByteArray(image);

            if (GAPI.G_NO_ERROR != GAPI_LVDS.Call_G_Lvds_FrameGenerator_DisplayBuffer(portNumber, ref DispBuff, ref imageBytes[0]))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }
            else
            {
                Console.WriteLine("Display image buffer");
            }
            
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            // Write I2C registers

            GAPI_LVDS.G_Lvds_Common_Data_Register_Write_Cmd_t WriteData = new GAPI_LVDS.G_Lvds_Common_Data_Register_Write_Cmd_t();

            WriteData.CmdFlags = 0x02;
            WriteData.RegisterAddress = 0x4F;
            WriteData.DeviceAddress = 0xC;
            WriteData.RegisterValue = 0x00;
            WriteData.NumberOfRegisters = 0x01; // only used if flag "EXTENDED_DATA" or "REGISTER_BLOCK" is set
            WriteData.Data = new Byte[256];
            WriteData.Data[0] = 0x00;              // only used if flag "EXTENDED_DATA" or "REGISTER_BLOCK" is set


            if (GAPI.G_NO_ERROR != GAPI_LVDS.Call_G_Lvds_Common_Data_Register_Write(portNumber, ref WriteData))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }
            else
            {
                Console.WriteLine("Registers written");
            }
            
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            // Read I2C registers with STOP between Write and read 
            // G_Lvds_Common_Data_I2cTransferEx

            // Send command to device read - address (here 0x71)

            GAPI_LVDS.G_Lvds_Common_Data_I2cTransferEx_Cmd_t SendData = new GAPI_LVDS.G_Lvds_Common_Data_I2cTransferEx_Cmd_t();

            SendData.CmdFlags = 0x01;
            SendData.NumberOfTxBytes = 2;
            SendData.NumberOfRxBytes = 0;
            SendData.TxData = new Byte[256];
            SendData.TxData[0] = 0xE2;      // DeviceAddress<<1
            SendData.TxData[1] = 0x04;      // Register Address

            GAPI_LVDS.G_Lvds_Common_Data_I2cTransferEx_Rsp_t ReadData = new GAPI_LVDS.G_Lvds_Common_Data_I2cTransferEx_Rsp_t();

            ReadData.RspFlags = 0;
            ReadData.NumberOfAckBytes = 0;
            ReadData.NumberOfRxBytes = 0;
            ReadData.RxData = new Byte[256];


            if (GAPI.G_NO_ERROR != GAPI_LVDS.Call_G_Lvds_Common_Data_I2cTransferEx(portNumber, ref SendData, ref ReadData))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }
            else
            {
                Console.WriteLine("Send Registers");
            }

            Thread.Sleep(100);

            // read from device reading - address (here 0x72)

            SendData.NumberOfTxBytes = 1;
            SendData.NumberOfRxBytes = 4;
            SendData.TxData[0] = 0xE3;      // (Device Address<<1)+1
            SendData.TxData[1] = 0x00;      // Register Address

            if (GAPI.G_NO_ERROR != GAPI_LVDS.Call_G_Lvds_Common_Data_I2cTransferEx(portNumber, ref SendData, ref ReadData))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }
            else
            {
                Console.WriteLine("Read Registers");
                for (UInt32 i = 0; i < SendData.NumberOfRxBytes; i++)
                {
                    Console.WriteLine(ReadData.RxData[i]);
                }
            }
            
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            // Last function!
            // Common_CloseInterface

            if (GAPI.G_NO_ERROR != GAPI_Common.Call_G_Common_CloseInterface(portNumber))
            {
                result_str = GAPI.Call_G_GetLastErrorDescription();
                Console.WriteLine(result_str);
            }

            Console.WriteLine("PortNumber " + portNumber + " closed");

            Console.WriteLine("Done");
            Console.ReadKey();
        }
    }
}
